/* cdw version 0.1.5
 * Copyright (C) 2002 Varkonyi Balazs
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
#include <stdio.h>
#include <ncurses.h>
#include <menu.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <dirent.h>
#include <libintl.h>
#include <locale.h>
#include <nl_types.h>
#include "thread.h"
#include "options.h"
#include "log.h"
#include "config.h"

void freemenu(MENU *name, ITEM **item, int number);
int one(struct dirent *unused);
void load_dir(char *dir, int width, int height);
void put_addwin(int width, int height, int y, int x);
void put_mainform();
int dirsize(char *dir);
int select_window(bool real);
void add_to_dic();

ITEM **items, **cditems, **fileitems;
MENU *menu, *cdmenu, *filemenu;
WINDOW *menuwin, *mainwin, *addwin, *select_sub, *menuwin_sub;
int choices, i, lines, cols, c, n, num;
float size;
char homedir[256], *TITLE;;
struct conf config;
int DEBUGMSG=0;

struct curses_menu {
    char add[20];
    char del[20];
    char cre[20];
    char cpi[20];
    char wri[20];
    char wau[20];
    char cdc[20];
    char blk[20];
    char opt[20];
    char qut[20];
    char nul[1];
} buttons;

void after_event(char *message, int adddic){
    if ( strcmp(config.showlog, "1")==0 )
        log_window(message, config.logfile, 0);
    if ( (strcmp(config.autodic, "1")==0) && (adddic==1) )
	add_to_dic();
    if ( strcmp(config.eject, "1")==0 ){
        eject_tray(config.cdrwdevice);
    }
}

void add_to_dic()
{
    char cdname[25], category[25], command[255];
    int ret;

#ifdef DIC        
    strcpy(cdname, "\0");
    strcpy(category, "none\0");
    if (input_volume((char *)gettext("CD name"), (char *)gettext("Enter CD name:"), cdname)==1) {
        ret=input_volume((char *)gettext("Category"), (char *)gettext("Enter category:"), category);
        if ( ret!=0 ) {
	    sprintf(command,"%s -d %s --category %s add %s", DIC, config.dicdisk, category, cdname);
	    if ( strcmp(cdname, "")!=0 ) run_command(command,(char *)gettext("Add CD to database"), 0, 1);
	}
    }
#endif
}

void freemenu(MENU *name, ITEM **item, int number)
{
    unpost_menu(name);
}

void fill_info()
{   wrefresh(menuwin_sub);
    if (strcmp(config.fromimage, "1")==0)
        mvwprintw(menuwin_sub, LINES-10, 2, (char *)gettext("Write from image"));
    else 
	mvwprintw(menuwin_sub, LINES-10, 2, (char *)gettext("Write direct    "));
    mvwprintw(menuwin_sub, LINES-9, 2, (char *)gettext("Speed: %sX "), config.speed);
    if (strcmp(config.multi, "1")==0)
	mvwprintw(menuwin_sub, LINES-8, 2, (char *)gettext("Multi-session "));
    else 
	mvwprintw(menuwin_sub, LINES-8, 2, (char *)gettext("Single session"));
    wrefresh(menuwin_sub);
}

int one(struct dirent *unused)
{
    if (strcmp(unused->d_name,".") && strcmp(unused->d_name,"..")) {
	return 1;
    }
    else
	return 0;
}

void load_dir(char *dir, int width, int height)
{
    struct dirent **eps;
    struct stat stbuf;
    char name[1000];
    
    int pos=0;
    n = scandir(dir, &eps, 0, alphasort);
    fileitems = (ITEM **)calloc(n, sizeof(ITEM *));
    if (n>0) {
	for (i=0; i<n; ++i) {
	    memset(name, '\0', sizeof(name));
	    strcpy(name, "/");
	    strcat(name, dir);
	    strcat(name, "/");
	    strcat(name, eps[i]->d_name);
	    if ( lstat(name, &stbuf) == -1) {
		endwin();
		fprintf(stderr, gettext("File: %s not found!"), name);
		exit(-1);
	    }
	    if ( (stbuf.st_mode& S_IFMT) == S_IFDIR )
		fileitems[pos++]=new_item(eps[i]->d_name," DIR ");
	}
	for (i=0; i<n; ++i) {
	    memset(name, '\0', sizeof(name));
	    strcpy(name, "/");
	    strcat(name, dir);
	    strcat(name, "/");
	    strcat(name, eps[i]->d_name);
	    if ( lstat(name, &stbuf) == -1) {
		endwin();
		fprintf(stderr, gettext("File: %s not found!"), name);
		exit(-1);
	    }
	    if ( (stbuf.st_mode& S_IFMT) != S_IFDIR )
		fileitems[pos++]=new_item(eps[i]->d_name,"");
	}
    }
    fileitems[n]=(ITEM *)NULL;
    filemenu = new_menu((ITEM **)fileitems);
    set_menu_win(filemenu, addwin);
    set_menu_sub(filemenu, derwin(addwin, width-2, height-2, 1, 1));
    set_menu_format(filemenu, width-2, 1);
    set_menu_fore(filemenu, COLOR_PAIR(2) | A_REVERSE);
    set_menu_back(filemenu, COLOR_PAIR(2));
    post_menu(filemenu);
    box(addwin, 0, 0);
    mvwprintw(addwin, width-1, 2, "( %s )",dir);
    wattrset(mainwin, A_REVERSE | COLOR_PAIR(2));    
    mvwaddch(addwin, 0, 2, ACS_RTEE);
    mvwprintw(addwin, 0, 3, " %s ", gettext("Add files"));
    waddch(addwin, ACS_LTEE);
    wrefresh(addwin);    
}

void put_addwin(int width, int height, int y, int x)
{
    char path[2000], linkfile[2000], newlink[2000];
    struct stat stbuf;
    char name[1000];
    char *pos;

    addwin=newwin(width,height,y,x);
    keypad(addwin, TRUE);
    wbkgd(addwin, A_NORMAL | COLOR_PAIR(2));
    wattrset(addwin, A_NORMAL | COLOR_PAIR(2));
    box(addwin, 0, 0);
    mvwaddch(addwin, 0, 2, ACS_RTEE);
    mvwprintw(addwin, 0, 3, " %s ", gettext("Add files"));
    waddch(addwin, ACS_LTEE);
    sprintf(path,"%s",getenv("HOME"));
    load_dir(path, width, height);
    while ( (c=wgetch(addwin))!=27 ){
	    switch(c){
		// First item
    		case KEY_HOME:
		    menu_driver(filemenu, REQ_FIRST_ITEM);
		    break;
		// Last item
		case KEY_END:
		    menu_driver(filemenu, REQ_LAST_ITEM);
		    break;
		// Next item
		case KEY_DOWN:
		    menu_driver(filemenu, REQ_DOWN_ITEM);
		    break;
		// Prev item
		case KEY_UP:
		    menu_driver(filemenu, REQ_UP_ITEM);
		    break;
		// Chdir
		case 10:
		    memset(name, '\0', sizeof(name));
		    strcpy(name, path);
		    strcat(name, "/");
		    strcat(name, item_name(current_item(filemenu)) );
		    if ( stat(name, &stbuf) == -1) {
			endwin();
			fprintf(stderr, gettext("File: %s not found!"), name);
			exit(-1);
		    }
		    if ( (stbuf.st_mode& S_IFMT) == S_IFDIR ) {
			strcpy(linkfile, path);
			if ( strcmp(item_name(current_item(filemenu)),".")==0 ){
			    strcpy(path,"/\0");
			}
			else {
			    if ( strcmp(item_name(current_item(filemenu)),"..")==0 ){
				if ( (pos=(char *)strrchr(&path, '/')) != NULL )
				    memset(pos,'\0',1);
				if ( strlen(path)<=0 ) 
				    strcat(path, "/");
			    } else {
				if ( path[strlen(path)-1]!='/' ) 
				    strcat(path, "/");
				strcat(path, item_name(current_item(filemenu)));
			    }
			}
			if ( access(path, R_OK | X_OK)==-1 ){
			    strcpy(path, linkfile);
			} else {
			    freemenu(filemenu, fileitems, n-1);
			    load_dir(path, width, height);
			}
		    }
		    break;
		// Add item
		case ' ':
		    strcpy(linkfile, path);
		    strcat(linkfile,"/");
		    strcat(linkfile,item_name(current_item(filemenu)));
		    strcpy(newlink,homedir);
		    strcat(newlink,"/");
		    strcat(newlink,item_name(current_item(filemenu)));
		    symlink(linkfile, newlink);
		    box(addwin, 0, 0);
		    mvwprintw(addwin, width-1, 2, "( %s )",path);
		    wattrset(mainwin, A_REVERSE | COLOR_PAIR(2));    
		    mvwaddch(addwin, 0, 2, ACS_RTEE);
		    mvwprintw(addwin, 0, 3, " %s ", gettext("Add files"));
		    waddch(addwin, ACS_LTEE);
		    select_window(TRUE);
		    menu_driver(filemenu, REQ_DOWN_ITEM);
		    break;
	    }
	    wrefresh(addwin);
    }
    freemenu(filemenu, fileitems, n);
    delwin(menu_sub(filemenu));
    delwin(menu_win(filemenu));
    delwin(addwin);
}


void put_mainform()
{
    mainwin=newwin(LINES-1,COLS-1,0,0);
    keypad(mainwin, TRUE);
    wbkgd(mainwin, A_NORMAL | COLOR_PAIR(1));
    box(mainwin, 0, 0);
    mvwaddch(mainwin, 2, 0, ACS_LTEE);
    mvwhline(mainwin, 2, 1, ACS_HLINE, COLS-3);
    mvwaddch(mainwin, 2, COLS-2, ACS_RTEE);
    wattrset(mainwin, A_BOLD | COLOR_PAIR(3));
    mvwprintw(mainwin, 1, (COLS / 2) - (strlen(TITLE) / 2), TITLE);
    wattrset(mainwin, A_NORMAL | COLOR_PAIR(1));

    // *** Main Menu ***
    strcpy(buttons.add, gettext(" Add files      "));
    strcpy(buttons.del, gettext(" Delete files   "));
    strcpy(buttons.cre, gettext(" Create image   "));
    strcpy(buttons.cpi, gettext(" Copy CD image  "));
    strcpy(buttons.wri, gettext(" Write data CD  "));
    strcpy(buttons.wau, gettext(" Write audio CD "));
    strcpy(buttons.cdc, gettext(" Copy data CD   "));
    strcpy(buttons.blk, gettext(" Blank CD-RW    "));
    strcpy(buttons.opt, gettext(" Options        "));
    strcpy(buttons.qut, gettext(" Quit           "));
    
    items = (ITEM **)calloc(11, sizeof(ITEM *));
    
    items[0]=new_item(buttons.add, "");
    items[1]=new_item(buttons.del, "");
    items[2]=new_item(buttons.cre, "");
    items[3]=new_item(buttons.cpi, "");
    items[4]=new_item(buttons.wri, "");
    items[5]=new_item(buttons.wau, "");
    items[6]=new_item(buttons.cdc, "");
    items[7]=new_item(buttons.blk, "");
    items[8]=new_item(buttons.opt, "");
    items[9]=new_item(buttons.qut, "");
    items[10]=new_item((char *)0, "");
    // *****************
    
    menu = new_menu((ITEM **)items);
    set_menu_win(menu, mainwin);
    set_menu_sub(menu, derwin(mainwin, LINES-8, 26, 5, 3));
    set_menu_fore(menu, COLOR_PAIR(4) | A_REVERSE);
    set_menu_back(menu, COLOR_PAIR(4));
    set_menu_mark(menu, "");
    select_sub=derwin(mainwin, LINES-5, COLS-29, 3, 27);
    menuwin_sub=derwin(mainwin, LINES-5, 26, 3, 1);
    box(derwin(mainwin, LINES-5, 26, 3, 1),0,0);
    box(select_sub,0,0);
    mvwaddch(menuwin_sub, 0, 2, ACS_RTEE);
    mvwprintw(menuwin_sub, 0, 3," %s ", gettext("Menu"));
    waddch(menuwin_sub, ACS_LTEE);
    mvwaddch(menuwin_sub, LINES-11, 0, ACS_LTEE);
    mvwhline(menuwin_sub, LINES-11, 1, ACS_HLINE, 24);
    mvwaddch(menuwin_sub, LINES-11, 25, ACS_RTEE);
    mvwprintw(menuwin_sub, LINES-11, 2,"( %s )", gettext("Info"));
    fill_info();

    mvwaddch(select_sub, 0, 2, ACS_RTEE);
    mvwprintw(select_sub, 0, 3," %s ",gettext("Selected files"));
    waddch(select_sub, ACS_LTEE);
    wattrset(select_sub, A_NORMAL | COLOR_PAIR(1));
    mvwaddch(select_sub, LINES-11, 0, ACS_LTEE);
    mvwhline(select_sub, LINES-11, 1, ACS_HLINE, COLS-31);
    mvwaddch(select_sub, LINES-11, COLS-30, ACS_RTEE);
    mvwaddch(select_sub, LINES-9, 0, ACS_LTEE);
    mvwhline(select_sub, LINES-9, 1, ACS_HLINE, COLS-31);
    mvwaddch(select_sub, LINES-9, COLS-30, ACS_RTEE);

    wattrset(mainwin, A_BOLD | COLOR_PAIR(2));
    post_menu(menu);
    wrefresh(mainwin);
}

int dirsize(char *dir)
{
    struct dirent **eps;
    struct stat *finfo;
    int n, ok, err, type, cnt;
    char *dname, *ch, ftime[256];

    n = scandir(dir, &eps, one, alphasort);
    if (n>=0) {
	for (cnt=0; cnt<n; ++cnt){
    	    ch=(char *) malloc(strlen(eps[cnt]->d_name)+strlen(dir)+4,sizeof(char));
	    sprintf(ch,"%s/%s",dir,eps[cnt]->d_name);
	    finfo=(struct stat *)malloc(sizeof(finfo)+100);
	    ok=lstat(ch,finfo);
	    if ( (eps[cnt]->d_type=DT_DIR) ) 
		dirsize(ch);
	    if (ok=stat(ch,finfo)!=0);
	    else {
		size+=finfo->st_size;
	    }
	    free(finfo);
    	    free(ch);
	}
    }
    return size;
}

void delete_files()
{
    int c;
    char filetodelete[200]="\0";
    
    while ( ((c=wgetch(mainwin))!=27) && (num>0) ){
	switch(c){
	    case KEY_HOME:
		menu_driver(cdmenu, REQ_FIRST_ITEM);
		break;
	    case KEY_END:
		menu_driver(cdmenu, REQ_LAST_ITEM);
		break;
	    case KEY_DOWN:
		menu_driver(cdmenu, REQ_DOWN_ITEM);
		break;
	    case KEY_UP:
		menu_driver(cdmenu, REQ_UP_ITEM);
		break;
	    case KEY_DC:
    		strcpy(filetodelete, homedir);
		strcat(filetodelete,"/");
		strcat(filetodelete,item_name(current_item(cdmenu)));
		unlink(filetodelete);
		freemenu(cdmenu, cditems, num+1);
		select_window(TRUE);
		set_menu_fore(cdmenu, COLOR_PAIR(1)|A_REVERSE);
		fflush(stdin);
		break;
	}
	wrefresh(derwin(mainwin, LINES-8, COLS-31, 4, 28));
    }
}

void size_selector()
{
    WINDOW *volwin, *edit;
    int ch=0;
    char cdsize[20];
    
    volwin=newwin(10,30,(LINES-10)/2,(COLS-30)/2);
    wbkgd(volwin, COLOR_PAIR(2));
    werase(volwin);
    box(volwin,0,0);
    mvwaddch(volwin, 0, 2, ACS_RTEE);
    mvwprintw(volwin, 0, 3, " %s ", (char *)gettext("CD size"));
    waddch(volwin, ACS_LTEE);
    mvwprintw(volwin, 2, 3, (char *)gettext("Select CD size"));
    mvwprintw(volwin, 4, 5, "1 - 74 Min (650 MB)");
    mvwprintw(volwin, 5, 5, "2 - 80 Min (700 MB)");
    mvwprintw(volwin, 6, 5, "3 -");
    edit=derwin(volwin, 1, 4, 6, 9);
    wbkgd(edit, COLOR_PAIR(6));
    werase(edit);
    sprintf(cdsize,"%d",config.cdsize);
    mvwprintw(edit, 0, 0, cdsize);
    if ( config.cdsize == 657 )
	mvwaddch(volwin, 4, 3, '*');
    else if ( config.cdsize == 702 )
	mvwaddch(volwin, 5, 3, '*');
    else {
	mvwaddch(volwin, 6, 3, '*');
    }
    wrefresh(edit);
    wmove(volwin, 9,29);
    while ( (ch!=27) && (ch!=10) ){
	ch=wgetch(volwin);
	switch(ch){
	    case '1':
		config.cdsize=657;
		break;
	    case '2':
		config.cdsize=702;
		break;
	    case '3':
		sprintf(cdsize,"%d",config.cdsize);
		inbox(edit, cdsize);
		config.cdsize=atoi(cdsize);
		break;
	}
	mvwaddch(volwin, 4, 3, ' ');
	mvwaddch(volwin, 5, 3, ' ');
	mvwaddch(volwin, 6, 3, ' ');
	if ( config.cdsize == 657 )
    	    mvwaddch(volwin, 4, 3, '*');
	else if ( config.cdsize == 702 )
    	    mvwaddch(volwin, 5, 3, '*');
	else
    	    mvwaddch(volwin, 6, 3, '*');
	wrefresh(volwin);
    }
    write_conf(config);
}

void space(int size)
{
    int i;
    float percent;

    for (i=1; i<40; i++)
	mvwaddch(mainwin,LINES-5,29+i, ' ');
    for (i=1; i<30; i++)
	mvwaddch(mainwin,LINES-4,29+i, ACS_BOARD);
    if (size<=config.cdsize) {
	percent=(size / ((float)config.cdsize / 100));
	for (i=1; i<(int)((percent*30)/100); i++)
	    mvwaddch(mainwin,LINES-4,29+i, ACS_BLOCK);
    }
    else {
	percent=(float)(100);
    	for (i=1; i<30; i++)
	    mvwaddch(mainwin,LINES-4,29+i, ACS_BLOCK);
    }
    if ( config.cdsize-size >= 0 ) 
	mvwprintw(mainwin,LINES-5,30, (char *)gettext("Used: %2.1f%%, Wasted: %d/%d MB"), percent, config.cdsize-size,config.cdsize);
    else
	mvwprintw(mainwin,LINES-5,30, (char *)gettext("Used: %2.1f%%, Wasted: %d MB"), percent, config.cdsize);
}

int cdda2wav()
{
    WINDOW *cddawin, *starttrack, *endtrack, *bitsperchn_win, *bitrate_win;
    FILE *fp;
    char *line;
    int start_track=0, end_track=0, max_track=0;
    int ch=0, bitspc, bitrate;
    char lame[2], tmp[18], encode[255], ogg[255], cdinfo[255], strk[2], etrk[2], bpc[2], stereo[2], echo[2], highq[2], brate[4];
    
    sprintf(cdinfo,"%s -D%s -J -H -g", CDDA2WAV, config.cdrom);
    run_command(cdinfo, gettext("Load CD info..."), 0, 0);

    if ( (fp=fopen(config.logfile,"r"))==NULL){
	endwin();
	fprintf(stderr, (char *)gettext("Cannot open file..."));
	exit(-1);
    }
    line=(char *)malloc(1024);

    while ( (line=fgets(line, 256, fp))!=NULL){
	// T01:      33  2:36.25 audio linear copydenied stereo title '' from ''
	if (sscanf(line, "T%d: %s %s audio %s", &max_track, tmp, tmp, cdinfo) == 4 ){
	    start_track=1;
	    end_track=max_track;
	}
    }
    cddawin=newwin(15,60,(LINES-15)/2,(COLS-60)/2);
    wbkgd(cddawin, COLOR_PAIR(2));
    werase(cddawin);
    box(cddawin,0,0);
    mvwaddch(cddawin, 0, 2, ACS_RTEE);
    mvwprintw(cddawin, 0, 3, " %s ", (char *)gettext("Grab Audio CD"));
    waddch(cddawin, ACS_LTEE);
    mvwprintw(cddawin, 14, 2, "( %s )", (char *)gettext("Press G to start or ESC to exit"));
    if (end_track!=0)
	mvwprintw(cddawin, 2, 2, "%s %d", (char *)gettext("Number of tracks:"), max_track);
    else {
	wattrset(cddawin, COLOR_PAIR(7));
	mvwprintw(cddawin, 2, 2, (char *)gettext("No audio CD!"), max_track);
    }
    wattrset(cddawin, COLOR_PAIR(2));
    mvwprintw(cddawin, 4, 2, (char *)gettext("<1>     Start track:"));
    mvwprintw(cddawin, 5, 2, (char *)gettext("<2>       End track:"));
    mvwprintw(cddawin, 7, 2, (char *)gettext("<S>             Stereo: [ ]"));
    mvwprintw(cddawin, 7, 32, (char *)gettext("<B>   Bits/Channel: "));
    mvwprintw(cddawin, 8, 2, (char *)gettext("<E> Echo to sound dev.: [ ]"));
#ifdef OGGENC
    wattrset(cddawin, COLOR_PAIR(2));
#else
    wattrset(cddawin, A_BOLD | COLOR_PAIR(9));
#endif
    mvwprintw(cddawin, 9, 2, (char *)gettext("<O>      Encode to Ogg: [ ]"));
    
    mvwprintw(cddawin, 9, 32, (char *)gettext("<H>   High Quality: [ ]"));
    mvwprintw(cddawin, 10, 32, (char *)gettext("<R>        Bitrate: "));
    bitrate_win=derwin(cddawin, 1, 4, 10, 52);
    wbkgd(bitrate_win, COLOR_PAIR(6));
    bitrate=atoi(config.bitrate); 
    mvwprintw(bitrate_win, 0, 0, "%d", bitrate);
    
#ifdef LAME
    wattrset(cddawin, COLOR_PAIR(2));
#else
    wattrset(cddawin, A_BOLD | COLOR_PAIR(9));
#endif
    mvwprintw(cddawin, 10, 2, (char *)gettext("<M>      Encode to Mp3: [ ]"));
    wattrset(cddawin, COLOR_PAIR(2));
  
    
    starttrack=derwin(cddawin, 1, 3, 4, 23);
    wbkgd(starttrack, COLOR_PAIR(6));
    werase(starttrack);
    endtrack=derwin(cddawin, 1, 3, 5, 23);
    wbkgd(endtrack, COLOR_PAIR(6));
    werase(endtrack);
    bitsperchn_win=derwin(cddawin, 1, 3, 7, 52);
    wbkgd(bitsperchn_win, COLOR_PAIR(6));
    werase(bitsperchn_win);
    mvwprintw(starttrack, 0, 0, "%d", start_track);
    mvwprintw(endtrack, 0, 0, "%d", end_track);
    bitspc=atoi(config.bitsperchn);
    mvwprintw(bitsperchn_win, 0, 0, "%d", bitspc);

    sprintf(stereo, "%s", config.stereo);
    sprintf(echo, "%s", config.echosound);
    sprintf(encode, "%s", config.encode);
    sprintf(lame, "%s", config.lame);
    sprintf(highq, "%s", config.highq);
    if ( strncmp(stereo,"1", strlen(stereo))==0)
	mvwprintw(cddawin, 7, 27, "X");
    if ( strncmp(echo,"1", strlen(echo))==0)
	mvwprintw(cddawin, 8, 27, "X");
    if ( strncmp(highq,"1", strlen(highq))==0)
	mvwprintw(cddawin, 9, 53, "X");
#ifdef OGGENC
    wattrset(cddawin, COLOR_PAIR(2));
    if ( strncmp(encode,"1", strlen(encode))==0)
	mvwprintw(cddawin, 9, 27, "X");
#endif
   
#ifdef LAME
   wattrset(cddawin, COLOR_PAIR(2));
   if ( strncmp(lame,"1", strlen(lame))==0)
	mvwprintw(cddawin, 10, 27, "X");
#endif
   
    wattrset(cddawin, COLOR_PAIR(2));
    while ( (ch=mvwgetch(cddawin,14,39)) != 27 ){
	switch(ch){
	    case '1':
		sprintf(strk,"%d",start_track);
		inbox(starttrack, strk);
		start_track=atoi(strk);
		if ( (start_track>max_track) || (start_track<=0) ){
		    wbkgd(starttrack, COLOR_PAIR(6));
		    werase(starttrack);
		    start_track=1;
		    mvwprintw(starttrack, 0, 0, "%d", start_track);
		}
		wrefresh(starttrack);
		break;
	    case '2':
		sprintf(etrk,"%d",end_track);
		inbox(endtrack, etrk);
		end_track=atoi(etrk);
		if ( (end_track>max_track) || (end_track<=0) ){
		    wbkgd(endtrack, COLOR_PAIR(6));
		    werase(endtrack);
		    end_track=max_track;
		    mvwprintw(endtrack, 0, 0, "%d", end_track);
		}
		wrefresh(endtrack);
		break;
	    case 's':
	    case 'S':
		if ( strncmp(stereo,"1",strlen(stereo))==0){
		    mvwprintw(cddawin, 7, 27, " ");
		    sprintf(stereo,"0");
		} else {
		    mvwprintw(cddawin, 7, 27, "X");
		    sprintf(stereo,"1");
		}
		break;
	    case 'h':
	    case 'H':
		if ( strncmp(highq,"1",strlen(highq))==0){
		    mvwprintw(cddawin, 9, 53, " ");
		    sprintf(highq,"0");
		} else {
		    mvwprintw(cddawin, 9, 53, "X");
		    sprintf(highq,"1");
		}
		break;
	    case 'b':
	    case 'B':
		sprintf(bpc,"%d",bitspc);
		inbox(bitsperchn_win, bpc);
		bitspc=atoi(bpc);
		if ( (bitspc!=16) && (bitspc!=12) && (bitspc!=8) ){
		    wbkgd(bitsperchn_win, COLOR_PAIR(6));
		    werase(bitsperchn_win);
		    bitspc=16;
		    mvwprintw(bitsperchn_win, 0, 0, "%d", bitspc);
		}
		wrefresh(bitsperchn_win);
		break;
	    case 'e':
	    case 'E':
		if ( strncmp(echo,"1", strlen(echo))==0){
		    mvwprintw(cddawin, 8, 27, " ");
		    sprintf(echo,"0");
		} else {
		    mvwprintw(cddawin, 8, 27, "X");
		    sprintf(echo,"1");
		}
		break;
#ifdef OGGENC		
	    case 'o':
	    case 'O':
		if ( strncmp(encode,"1", strlen(encode))==0){
		    mvwprintw(cddawin, 9, 27, " ");
		    sprintf(encode,"0");
		} else {
		    mvwprintw(cddawin, 9, 27, "X");
		    sprintf(encode,"1");
		    sprintf(lame, "0");
		    mvwprintw(cddawin, 10, 27, " ");
		}
		break;
#endif
#ifdef LAME		
	    case 'm':
	    case 'M':
		if ( strncmp(lame,"1", strlen(lame))==0){
		    mvwprintw(cddawin, 10, 27, " ");
		    sprintf(lame,"0");
		} else {
		    mvwprintw(cddawin, 10, 27, "X");
		    sprintf(lame,"1");
		    sprintf(encode, "0");
		    mvwprintw(cddawin, 9, 27, " ");
	    
		}
		break;
#endif

	    case 'r':
	    case 'R':	
		sprintf(brate,"%d",bitrate);
		inbox(bitrate_win, brate);
		bitrate=atoi(brate);
		if ( (bitrate!=128) && (bitrate!=192) && (bitrate!=256) ){
		    wbkgd(bitrate_win, COLOR_PAIR(6));
		    werase(bitrate_win);
		    bitrate=192;
		    mvwprintw(bitrate_win, 0, 0, "%d", bitrate);
		}
		wrefresh(bitrate_win);
		break;
	    case 'g':
	    case 'G':
		if ( end_track!=0 ){
		    sprintf(config.bitsperchn, "%d", bitspc);
		    sprintf(config.stereo, "%s", stereo);
		    sprintf(config.echosound, "%s", echo);
		    sprintf(config.encode, "%s", encode);
		    sprintf(config.lame, "%s", lame);
		    sprintf(config.highq,"%s",highq);
		    sprintf(config.bitrate,"%d", bitrate);
		    write_conf(config);
		    sprintf(cdinfo, "%s ", CDDA2WAV);
		    if ( strncmp(stereo,"1",strlen(stereo))==0)
			sprintf(cdinfo, "%s -s", cdinfo);
		    else
			sprintf(cdinfo, "%s -m", cdinfo);
		    if ( strncmp(echo,"1", strlen(echo))==0){
			sprintf(cdinfo, "%s -e", cdinfo);
		    }
		    if ( strncmp(encode,"1", strlen(encode))==0){
#ifdef OGGENC			    
			wrefresh(derwin(mainwin,LINES-1,COLS-1,0,0));    
			if  ( strncmp(highq,"1", strlen(highq))==0){
			sprintf(cdinfo, "%s -D%s -t$T -b%d -g -H - | %s -q 8 -o %s/audio$T.ogg -; T=`echo $((T+1))`; done", cdinfo, config.cdrom, bitspc, OGGENC, config.audiodir);
			}
			else
			    
			sprintf(cdinfo, "%s -D%s -t$T -b%d -g -H - | %s -b%d -o %s/audio$T.ogg -; T=`echo $((T+1))`; done", cdinfo, config.cdrom, bitspc, OGGENC, bitrate, config.audiodir);
			sprintf(ogg, "T=%d; while [ $T -le %d ]; do %s", start_track, end_track, cdinfo);
			run_command(ogg, gettext("Grab Audio CD"), 0, 0);
#endif			
		    } else if ( strncmp(lame,"1", strlen(lame))==0){
#ifdef LAME			    
			wrefresh(derwin(mainwin,LINES-1,COLS-1,0,0));    
			if  ( strncmp(highq,"1", strlen(highq))==0){
			sprintf(cdinfo, "%s -D%s -t$T -b%d -g -H - | %s -h -V2 -b%d - %s/audio$T.mp3 -; T=`echo $((T+1))`; done", cdinfo, config.cdrom, bitspc, LAME, bitrate, config.audiodir);
			}
			else
			sprintf(cdinfo, "%s -D%s -t$T -b%d -g -H - | %s -b%d - %s/audio$T.mp3; T=`echo $((T+1))`; done", cdinfo, config.cdrom, bitspc, LAME, bitrate, config.audiodir);
			sprintf(ogg, "T=%d; while [ $T -le %d ]; do %s", start_track, end_track, cdinfo);
			run_command(ogg, gettext("Grab Audio CD"), 0, 0);
#endif			
		    
		    } else {
			wrefresh(derwin(mainwin,LINES-1,COLS-1,0,0));    
			sprintf(cdinfo, "%s -D%s -t%d+%d -b%d -g -H -B %s/audio.", cdinfo, config.cdrom, start_track, end_track, bitspc, config.audiodir);
			run_command(cdinfo, gettext("Grab Audio CD"), 0, 0);
		    }
		    wrefresh(derwin(cddawin,15,60,0,0));    
		    wrefresh(cddawin);
		}
		break;
	}
	wrefresh(cddawin);
    }
}

int select_window(bool real)
{
    struct dirent **cdfiles;
    int pos=0, datasize=0;
    
    if (real==TRUE) {
	num = scandir(homedir, &cdfiles, one, alphasort);
	cditems = (ITEM **)calloc(num+1, sizeof(ITEM *));
	if (num>0) {
    	    for (i=0; i<num; ++i) {
    		if ( cdfiles[i]->d_type==DT_DIR )
		    cditems[pos++]=new_item(cdfiles[i]->d_name," DIR");
	    }
	    for (i=0; i<num; ++i) {
		if ( cdfiles[i]->d_type!=DT_DIR )
	    	    cditems[pos++]=new_item(cdfiles[i]->d_name,"");
	    }
	}
	cditems[num]=(ITEM *)NULL;
	cdmenu = new_menu((ITEM **)cditems);
	set_menu_win(cdmenu, mainwin);
	set_menu_sub(cdmenu, derwin(mainwin, LINES-12, COLS-31, 4, 28));
	set_menu_format(cdmenu, LINES-12, 1);
	set_menu_fore(cdmenu, COLOR_PAIR(1));
	set_menu_back(cdmenu, COLOR_PAIR(1));
	post_menu(cdmenu);
    }
    select_sub=derwin(mainwin, LINES-8, COLS-29, 3, 27);
    wattrset(select_sub, A_NORMAL | COLOR_PAIR(1));
    mvwaddch(select_sub, LINES-11, 0, ACS_LTEE);
    mvwhline(select_sub, LINES-11, 1, ACS_HLINE, COLS-31);
    mvwaddch(select_sub, LINES-11, COLS-30, ACS_RTEE);
    wattrset(mainwin, A_BOLD | COLOR_PAIR(8));
    size=0;
    mvwprintw(mainwin,LINES-7,30,gettext("Size: %6d Mb in %d files"),dirsize(homedir)/1024/1024, num);
    space(size/1024/1024);
    wrefresh(mainwin);
}

int input_volume(char *title, char *label, char *prop)
{
    WINDOW *volwin, *edit;
    int ret;
    
    volwin=newwin(10,30,(LINES-10)/2,(COLS-30)/2);
    wbkgd(volwin, COLOR_PAIR(2));
    werase(volwin);
    box(volwin,0,0);
    mvwaddch(volwin, 0, 2, ACS_RTEE);
    mvwprintw(volwin, 0, 3, " %s ", (char *)gettext(title));
    waddch(volwin, ACS_LTEE);
    mvwprintw(volwin, 3, 3, (char *)gettext(label));
    wrefresh(volwin);
    edit=derwin(volwin, 1, 24, 5, 3);
    wbkgd(edit, COLOR_PAIR(6));
    werase(edit);
    wrefresh(edit);
    ret=inbox(edit, prop);
    write_conf(config);
    return ret;
}

int main(int argc, int argv[])
{
    char command[255];
    char buffer[80];
    FILE *fp;
    int fd[2], title_len, ret;
    char catfile[256];
    char cdname[25], category[25];
    
    /* Read config */
    if ( argc>1 ) {
	printf(gettext("Usage: %s (no parameter)\n"),argv[0]);
	exit(-1);
    }
    sprintf(homedir,"%.225s/.cdw",getenv("HOME"));
    mkdir(homedir,S_IRWXU);
    config=read_conf();

    setlocale(LC_ALL, "");
    textdomain(PACKAGE);
    bindtextdomain(PACKAGE, LOCALEDIR);

    title_len=strlen(PACKAGE)+2+strlen(VERSION)+9;
    TITLE=(char *)malloc(title_len,sizeof(char));
    sprintf(TITLE, "::: %s-%s :::",PACKAGE, VERSION);
    initscr();
    start_color();
    cbreak();
    noecho();
    keypad(stdscr, TRUE);

    // colors
    init_pair(1, COLOR_CYAN, COLOR_BLUE);
    init_pair(2, COLOR_BLUE, COLOR_WHITE);
    init_pair(3, COLOR_RED, COLOR_BLUE);
    init_pair(4, COLOR_WHITE, COLOR_BLUE);
    init_pair(5, COLOR_BLACK, COLOR_BLUE);
    init_pair(6, COLOR_WHITE, COLOR_BLACK);
    init_pair(7, COLOR_RED, COLOR_WHITE);
    init_pair(8, COLOR_YELLOW, COLOR_BLUE);
    init_pair(9, COLOR_BLACK, COLOR_WHITE);
    init_pair(10, COLOR_WHITE, COLOR_RED);

    lines=LINES-2;
    if ( ( LINES<24 ) || ( COLS<79 ) ) {
	endwin();
	printf(gettext("Needed min 80x25 terminal!"));
	exit(-1);
    }
    
    put_mainform();
    select_window(TRUE);
    while ( (c=wgetch(mainwin))!='q' ){
	switch(c){
	    
	    // -------- HOT KEYS --------    
	
	    case 'e':
	    case 'E':
		// eject tray
		eject_tray(config.cdrwdevice);
		break;
	    case 'g':
	    case 'G':
		// cdda2wav
		cdda2wav();
    		wrefresh(derwin(mainwin,LINES-1,COLS-1,0,0));    
		select_window(TRUE);
		break;
	    case KEY_F(1):
		// Show readme
		sleep(2);
		sprintf(command, "%s/README.gz", DOCDIR);
		log_window((char *)gettext("Help"), command, 1);
		wrefresh(derwin(mainwin,LINES-1,COLS-1,0,0));    
		select_window(TRUE);
		break;
	    case KEY_F(2):
		// Edit volume ID
		input_volume((char *)gettext("Volume ID"), (char *)gettext("Enter volume label:"), config.volumeid);
    		wrefresh(derwin(mainwin,LINES-1,COLS-1,0,0));    
		select_window(TRUE);
		break;
	    case KEY_F(3):
		// Edit other parameters
		input_volume((char *)gettext("CD record parameters"), (char *)gettext("Enter parameter:"), config.other);
    		wrefresh(derwin(mainwin,LINES-1,COLS-1,0,0));    
		select_window(TRUE);
		break;
	    case KEY_F(4):
		// CD size select
		size_selector();
    		wrefresh(derwin(mainwin,LINES-1,COLS-1,0,0));    
		select_window(TRUE);
		break;
	    case 'L':
	    case 'l':
	    case KEY_F(5):
		// Show last log
		log_window((char *)gettext("Last log"), config.logfile, 0);
		wrefresh(derwin(mainwin,LINES-1,COLS-1,0,0));    
		select_window(TRUE);
		break;
	    case KEY_F(6):
		// Show CD info
		sprintf(command,"%s dev=%s -atip", CDRECORD, config.scsi);
		run_command(command,gettext("Load CD info..."), 0, 0);
		log_window((char *)gettext("CD info"), config.logfile, 0);
		wrefresh(derwin(mainwin,LINES-1,COLS-1,0,0));    
		select_window(TRUE);
		break;
/*	    case KEY_F(9):
		dialogbox("This is a sample message. Show in a simple\ndialog box\n\nOk?", "Dialog box", 3);
		wrefresh(derwin(mainwin,LINES-1,COLS-1,0,0));    
		select_window(TRUE);
		break;*/

	    case KEY_F(10):
		// Show GPL
		sprintf(command, "%s/LICENSE.gz", DOCDIR);
		log_window((char *)gettext("License"), command, 1);
		wrefresh(derwin(mainwin,LINES-1,COLS-1,0,0));    
		select_window(TRUE);
		break;
#ifdef DIC
	    case 'A':
	    case 'a':
		// Add disk to dic database
		add_to_dic();
		wrefresh(derwin(mainwin,LINES-1,COLS-1,0,0));    
		select_window(TRUE);
		break;
#endif
	    // -------- MENU --------

	    case KEY_HOME:
		menu_driver(menu, REQ_FIRST_ITEM);
		break;
	    case KEY_END:
		menu_driver(menu, REQ_LAST_ITEM);
		break;
	    case KEY_DOWN:
		menu_driver(menu, REQ_DOWN_ITEM);
		break;
	    case KEY_UP:
		menu_driver(menu, REQ_UP_ITEM);
		break;
	    case 10:
		// Add files
		if ( item_index(current_item(menu)) == 0 ){
		    put_addwin(15,60,(LINES-15)/2,(COLS-60)/2);
		    wrefresh(derwin(mainwin,LINES-1,COLS-1,0,0));    
		    select_window(TRUE);
		}
		// Delete files
		if ( item_index(current_item(menu)) == 1 ){
		    // If cd files not empty!
		    if ( num>0 ) {
			set_menu_fore(cdmenu, COLOR_PAIR(1) | A_REVERSE);
			set_menu_fore(menu, COLOR_PAIR(4));
			wrefresh(mainwin);
			delete_files();
			set_menu_fore(cdmenu, COLOR_PAIR(1));
			set_menu_fore(menu, COLOR_PAIR(4) | A_REVERSE);
		    }
		}
		// Create image
		if ( (item_index(current_item(menu)) == 2) && (num>0) ){
		    int ret=1;
		    if ( strcmp(config.showvol,"1")==0 ){
			ret=input_volume((char *)gettext("Volume ID"), (char *)gettext("Enter volume label:"), config.volumeid);
    			wrefresh(derwin(mainwin,LINES-1,COLS-1,0,0));    
			select_window(TRUE);
		    }
		    if (ret==1) {
			sprintf(command,"%s -f", MKISOFS);
			if ( strcmp(config.joliet, "1")==0 ) {
			    strcat(command," -J");
			}
			if ( strcmp(config.rockridge, "1")==0 ) {
			    strcat(command," -r");
			}
			if ( strlen(config.volumeid) == 0){
			    strcpy(config.volumeid, "cdrom");
			}
			sprintf(command,"%s -V\"%s\" -o %s %s", command, config.volumeid, config.tempdir, homedir);
			run_command(command,(char *)gettext("Create image"), 0, 1);
		    }
		    wrefresh(derwin(mainwin,LINES-1,COLS-1,0,0));    
		    select_window(FALSE);
		}
		// Copy image
		if ( (item_index(current_item(menu)) == 3) ){
		    copy_image(gettext("Copy CD image"));
		    wrefresh(derwin(mainwin,LINES-1,COLS-1,0,0));    
		    select_window(FALSE);
		    wrefresh(derwin(mainwin,LINES-1,COLS-1,0,0));    
		    select_window(FALSE);
		}
		// Write image
		if ( (item_index(current_item(menu)) == 4) && (access(config.tempdir, R_OK)!=-1) && ( strcmp(config.fromimage, "1")==0 ) ){
		    sprintf(command,"%s -v speed=%s dev=%s %s",CDRECORD,config.speed,config.scsi,config.other);
		    if ( strcmp(config.dao, "1")==0 ){
			strcat(command, " -dao");
		    }
		    if ( strcmp(config.dummy, "1")==0 ){
			strcat(command, " -dummy");
		    }
		    if ( strcmp(config.burnproof, "1")==0 ){
			strcat(command, " -driveropts=burnproof");
		    }
		    sprintf(command,"%s -data %s",command, config.tempdir);
		    run_command(command, gettext("Write image"), 0, 1);
		    after_event("Write image log", 1);
		    wrefresh(derwin(mainwin,LINES-1,COLS-1,0,0));    
		    select_window(FALSE);
		}
		// Write direct
		if ( (item_index(current_item(menu)) == 4) && ( strcmp(config.fromimage, "0")==0 )){
		    int ismulti=0, ret=1;
		    if ( strcmp(config.showvol,"1")==0 ){
			ret=input_volume("Volume ID", "Enter volume label:", config.volumeid);
    			wrefresh(derwin(mainwin,LINES-1,COLS-1,0,0));    
			select_window(TRUE);
		    }
		    if (ret==1) {
			sprintf(command,"%s -f", MKISOFS);
			if ( strcmp(config.joliet, "1")==0 ) {
			    strcat(command," -J");
			}
			if ( strcmp(config.rockridge, "1")==0 ) {
			    strcat(command," -R");
			}
			if ( strcmp(config.usefulRR, "1")==0 ) {
			    strcat(command," -r");
			}
			if ( strcmp(config.multi, "1")==0 ) {
			    char msinfo[255];
			    char *line;
			    int track_start=0, track_end=0;
			    sprintf(msinfo,"%s dev=%s -msinfo", CDRECORD, config.scsi);
			    run_command(msinfo, gettext("Load CD info..."), 0, 0);
			    if ( (fp=fopen(config.logfile,"r"))==NULL){
				endwin();
				fprintf(stderr, (char *)gettext("Cannot open file..."));
				exit(-1);
			    }
			    line=(char *)malloc(1024);
			
			    while ( (line=fgets(line, 256, fp))!=NULL){
				if ( sscanf(line, "%d,%d", &track_start, &track_end) == 2 ){
				    wrefresh(mainwin);
				    ismulti=1;
				}
			    }
			    if (ismulti==1) {
				sprintf(command,"%s -C %d,%d -M %s",command, track_start, track_end, config.cdrwdevice);
			    }
			    fclose(fp);
			    free(line);
			}
			if ( strlen(config.volumeid) == 0){
			    strcpy(config.volumeid, "cdrom");
			}
			sprintf(command,"%s -V\"%s\" %s", command, config.volumeid, homedir);
			sprintf(command,"%s | %s -v speed=%s dev=%s %s",command, CDRECORD,config.speed,config.scsi,config.other);
			if ( strcmp(config.dao, "1")==0 ){
			    strcat(command, " -dao");
			}
			if ( strcmp(config.dummy, "1")==0 ){
			    strcat(command, " -dummy");
			}
			if ( strcmp(config.multi, "1")==0){
			    strcat(command, " -multi");
			}
			if ( strcmp(config.burnproof, "1")==0 ){
			    strcat(command, " -driveropts=burnproof");
			}
			sprintf(command,"%s -",command);
			wrefresh(mainwin);
			run_command(command, (char *)gettext("Write direct"), size, 1);
			after_event("Write direct log", 1);
		    }
		    wrefresh(derwin(mainwin,LINES-1,COLS-1,0,0));    
		    select_window(FALSE);
		}
		// Write Audio
		if ( (item_index(current_item(menu)) == 5) ){
		    int ret=1;
		    if ( strcmp(config.showvol,"1")==0 ){
			ret=input_volume("Volume ID", "Enter volume label:", config.volumeid);
    			wrefresh(derwin(mainwin,LINES-1,COLS-1,0,0));    
			select_window(TRUE);
		    }
		    if (ret==1) {
			sprintf(command,"%s -v speed=%s dev=%s -audio",CDRECORD,config.speed,config.scsi);
			if ( strcmp(config.pad, "1")==0 ){
			    strcat(command, " -pad");
			}
			if ( strcmp(config.dao, "1")==0 ){
			    strcat(command, " -dao");
			}
			if ( strcmp(config.dummy, "1")==0 ){
			    strcat(command, " -dummy");
			}
			if ( strcmp(config.burnproof, "1")==0 ){
			    strcat(command, " -driveropts=burnproof");
			}
			sprintf(command,"%s %s/*.wav",command,homedir);
			run_command(command, (char *)gettext("Write audio"), size, 1);
			after_event("Write audio log", 1);
		    }
		    wrefresh(derwin(mainwin,LINES-1,COLS-1,0,0));    
		    select_window(FALSE);
		}
		// Copy data CD
		if ( (item_index(current_item(menu)) == 6) && (config.cdrom!=config.cdrwdevice) ){
		    int in_fd;
		    if ( (in_fd = open("/dev/cdrom", O_RDONLY)) != -1 ){
			close(in_fd);
			sprintf(command,"%s -v speed=%s dev=%s %s",CDRECORD,config.speed,config.scsi,config.other);
			if ( strcmp(config.eject, "1")==0 ){
			    strcat(command, " -eject");
			}
			if ( strcmp(config.dummy, "1")==0 ){
			    strcat(command, " -dummy");
			}
			sprintf(command,"%s -isosize %s",command, config.cdrom);
			run_command(command, gettext("Copy data CD"), 0, 1);
			after_event("Copy data CD log", 1);
		wrefresh(derwin(mainwin,LINES-1,COLS-1,0,0));    
		select_window(FALSE);
		    } 
		}
		// Blank CD-RW
		if ( item_index(current_item(menu)) == 7){
		    sprintf(command,"%s -v speed=%s dev=%s blank=%s",CDRECORD,config.speed,config.scsi,config.blank);
		    run_command(command,gettext("Blank CD-RW"), 0, 1);
		    wrefresh(derwin(mainwin,LINES-1,COLS-1,0,0));    
		    select_window(FALSE);
		}
		// Options
		if ( item_index(current_item(menu)) == 8){
		    options_window();
		    config=read_conf();
		    wrefresh(derwin(mainwin,LINES-1,COLS-1,0,0));    
		    select_window(FALSE);
		    fill_info();
		}
		// Quit
		if ( item_index(current_item(menu)) == 9){
		    endwin();
		    exit(1);
		}
		break;
	}
	wrefresh(mainwin);
    }
    unpost_menu(menu);
    free_menu(menu);
    for(i = 0; i < choices; ++i)
        free_item(items[i]);
    endwin();
    return 1;
}
