#include <stdio.h>
#include <unistd.h>
#include <ctype.h>
#include <ncurses.h>
#include <nl_types.h>
#include "options.h"
#include "config.h"

struct optwins {
    WINDOW *main;
    WINDOW *scsidev;
    WINDOW *speed;
    WINDOW *tempdir;
    WINDOW *logfile;
    WINDOW *volumeid;
    WINDOW *cdrom;
    WINDOW *cdrwdevice;
    WINDOW *dicdisk;
    WINDOW *audiodir;
    WINDOW *bpc;
    WINDOW *bitrate;
} optwin;
struct conf config;

void clear_main_win()
{
    WINDOW *subwin;
    
    subwin=derwin(optwin.main, 18, 58, 1, 1);
    werase(subwin);
    wrefresh(subwin);
}

void write_conf(struct conf config)
{
    FILE *conffile;
    char home[256], opt[20], prop[256];

    sprintf(home,"%.225s/.cdw.conf", getenv("HOME"));
    if ( (conffile = fopen(home, "w")) != NULL ){
	fprintf(conffile,"################################\n");
	fprintf(conffile,"#    cdw configuration file    #\n");
	fprintf(conffile,"################################\n");
	fprintf(conffile,"scsi-dev=%s\n",config.scsi);
	fprintf(conffile,"cdrwdevice=%s\n",config.cdrwdevice);
	fprintf(conffile,"tempdir=%s\n",config.tempdir);
	fprintf(conffile,"speed=%s\n",config.speed);
	fprintf(conffile,"blank=%s\n",config.blank);
	fprintf(conffile,"logfile=%s\n",config.logfile);
	fprintf(conffile,"other=%s\n",config.other);
	fprintf(conffile,"eject=%s\n",config.eject);
	fprintf(conffile,"dao=%s\n",config.dao);
	fprintf(conffile,"joliet=%s\n",config.joliet);
	fprintf(conffile,"rockridge=%s\n",config.rockridge);
	fprintf(conffile,"dummy=%s\n",config.dummy);
	fprintf(conffile,"volumeid=%s\n",config.volumeid);
	fprintf(conffile,"cdrom=%s\n",config.cdrom);
	fprintf(conffile,"usefulRR=%s\n",config.usefulRR);
	fprintf(conffile,"showlog=%s\n",config.showlog);
	fprintf(conffile,"showvol=%s\n",config.showvol);
	fprintf(conffile,"cdsize=%d\n",config.cdsize);
	fprintf(conffile,"pad=%s\n",config.pad);
	fprintf(conffile,"WriteFromImage=%s\n",config.fromimage);
	fprintf(conffile,"multi=%s\n",config.multi);
	fprintf(conffile,"burnproof=%s\n",config.burnproof);
	fprintf(conffile,"dicdisk=%s\n",config.dicdisk);
	fprintf(conffile,"autodic=%s\n",config.autodic);
	fprintf(conffile,"audiodir=%s\n",config.audiodir);
	fprintf(conffile,"stereo=%s\n",config.stereo);
	fprintf(conffile,"bitsperchn=%s\n",config.bitsperchn);
	fprintf(conffile,"echosound=%s\n",config.echosound);
	fprintf(conffile,"encode=%s\n",config.encode);
	fprintf(conffile,"lame=%s\n",config.lame);
	fprintf(conffile,"highq=%s\n",config.highq);
	fprintf(conffile,"bitrate=%s\n",config.bitrate);
	fclose(conffile);
    }
}

struct conf read_conf()
{
    FILE *conffile;
    char home[256], opt[20], prop[256];
    int line=0;
    
    strcpy(config.other,"\0");
    strcpy(config.scsi,"0,0,0\0");
    strcpy(config.cdrwdevice,"/dev/scd0\0");
    strcpy(config.tempdir,"/tmp/image.iso\0");
    strcpy(config.speed,"1\0");
    strcpy(config.blank,"fast\0");
    strcpy(config.logfile,"/var/log/cdw.log\0");
    strcpy(config.eject,"0\0");
    strcpy(config.dao,"0\0");
    strcpy(config.joliet,"0\0");
    strcpy(config.rockridge,"1\0");
    strcpy(config.dummy,"0\0");
    strcpy(config.volumeid,"cdrom\0");
    strcpy(config.cdrom,"/dev/cdrom\0");
    strcpy(config.usefulRR,"1\0");
    strcpy(config.showlog,"1\0");
    strcpy(config.showvol,"0\0");
    strcpy(config.pad,"1\0");
    strcpy(config.fromimage,"1\0");
    strcpy(config.multi,"0\0");
    strcpy(config.burnproof,"0\0");
    strcpy(config.dicdisk,"cdrom\0");
    strcpy(config.dicdisk,"0\0");
    strcpy(config.audiodir,"~/audio\0");
    strcpy(config.stereo,"1\0");
    strcpy(config.bitsperchn,"16\0");
    strcpy(config.echosound,"0\0");
    strcpy(config.encode,"0\0");
    strcpy(config.lame,"0\0");
    strcpy(config.highq,"1\0");
    strcpy(config.bitrate,"192\0");
    config.cdsize=657;
    sprintf(home,"%.225s/.cdw.conf", getenv("HOME"));
    if ( (conffile = fopen(home, "r")) == NULL ){
	conffile = fopen(home, "w");
	fprintf(conffile,"################################\n#    cdw configuration file    #\n");
	fprintf(conffile,"################################\nscsi-dev=0,0,0\ntempdir=/tmp/image.iso\nspeed=1\nblank=fast\n");
	fprintf(conffile,"logfile=/tmp/cdw.log\neject=0\ndao=0\njoliet=0\nrockridge=1\nusefulRR=1\ndummy=0\nvolumeid=cdrom\ncdrom=/dev/cdrom\n");
	fprintf(conffile,"showlog=0\nshowvol=0\ncdsize=657\npad=1\nWriteFromImage=1\nmulti=0\nburnproof=0\ncdrwdevice=/dev/scd0\ndicdisk=cdrom\nautodic=0\naudiodir=~/audio\n");
	fprintf(conffile,"stereo=1\nbitsperchn=16\nechosound=0\nencode=0\nlame=0\nhighq=1\nbitrate=192\n");
	fclose(conffile);
	conffile = fopen(home, "r");
    }
    while (NULL != fgets(home, 255, conffile)) {
	if ( sscanf(home, "%30[^=]=%[^\n]", opt, prop) == 2 ) {
	    if ( (strcmp(opt,"scsi-dev")) == 0 )
		strcpy(config.scsi, prop);
	    if ( (strcmp(opt,"cdrwdevice")) == 0 )
		strcpy(config.cdrwdevice, prop);
	    if ( (strcmp(opt,"tempdir")) == 0 )
		strcpy(config.tempdir, prop);
	    if ( (strcmp(opt,"speed")) == 0 )
		strcpy(config.speed, prop);
	    if ( (strcmp(opt,"other")) == 0 )
		strcpy(config.other, prop);
	    if ( (strcmp(opt,"blank")) == 0 )
		strcpy(config.blank, prop);
	    if ( (strcmp(opt,"logfile")) == 0 )
		strcpy(config.logfile, prop);
	    if ( (strcmp(opt,"eject")) == 0 )
		strcpy(config.eject, prop);
	    if ( (strcmp(opt,"dao")) == 0 )
		strcpy(config.dao, prop);
	    if ( (strcmp(opt,"joliet")) == 0 )
		strcpy(config.joliet, prop);
	    if ( (strcmp(opt,"rockridge")) == 0 )
		strcpy(config.rockridge, prop);
	    if ( (strcmp(opt,"dummy")) == 0 )
		strcpy(config.dummy, prop);
	    if ( (strcmp(opt,"volumeid")) == 0 )
		strcpy(config.volumeid, prop);
	    if ( (strcmp(opt,"cdrom")) == 0 )
		strcpy(config.cdrom, prop);
	    if ( (strcmp(opt,"usefulRR")) == 0 )
		strcpy(config.usefulRR, prop);
	    if ( (strcmp(opt,"showlog")) == 0 )
		strcpy(config.showlog, prop);
	    if ( (strcmp(opt,"showvol")) == 0 )
		strcpy(config.showvol, prop);
	    if ( (strcmp(opt,"cdsize")) == 0 )
		config.cdsize=atoi(prop);
	    if ( (strcmp(opt,"pad")) == 0 )
		strcpy(config.pad, prop);
	    if ( (strcmp(opt,"WriteFromImage")) == 0 )
		strcpy(config.fromimage, prop);
	    if ( (strcmp(opt,"multi")) == 0 )
		strcpy(config.multi, prop);
	    if ( (strcmp(opt,"burnproof")) == 0 )
		strcpy(config.burnproof, prop);
	    if ( (strcmp(opt,"dicdisk")) == 0 )
		strcpy(config.dicdisk, prop);
	    if ( (strcmp(opt,"autodic")) == 0 )
		strcpy(config.autodic, prop);
	    if ( (strcmp(opt,"audiodir")) == 0 )
		strcpy(config.audiodir, prop);
	    if ( (strcmp(opt,"stereo")) == 0 )
		strcpy(config.stereo, prop);
	    if ( (strcmp(opt,"bitsperchn")) == 0 )
		strcpy(config.bitsperchn, prop);
	    if ( (strcmp(opt,"echosound")) == 0 )
		strcpy(config.echosound, prop);
	    if ( (strcmp(opt,"encode")) == 0 )
		strcpy(config.encode, prop);
	    if ( (strcmp(opt,"lame")) == 0 )
		strcpy(config.lame, prop);
	    if ( (strcmp(opt,"highq")) == 0 )
		strcpy(config.highq, prop);
	    if ( (strcmp(opt,"bitrate")) == 0 )
		strcpy(config.bitrate, prop);
	    

	}
    }
    fclose(conffile);
    return config;
}

int inbox(WINDOW *input, char *init)
{
    int in_x, ch=0;
    char instr[256];

    keypad(input, TRUE);
    werase(input);
    strcpy(instr, init);
    if ( strlen(init)==0 )
	instr[0]='\0';
    else
	strcpy(instr, init);
    mvwprintw(input, 0, 0, init);
    in_x=strlen(instr);
    while ( (ch!=10) && (ch!=27) ) {
	ch=wgetch(input);
	if ( ch==27 ) {
	    strcpy(instr, init);
	}
	if ( ch==KEY_BACKSPACE && in_x>0) {
	    instr[in_x-1]='\0';
	    in_x--;
	    mvwaddch(input, 0, in_x, ' ');
	    wmove(input, 0, in_x);
	}
	if ( isprint(ch) ) {
	    waddch(input, ch);
	    instr[in_x]=ch;
	    instr[in_x+1]='\0';
	    in_x++;
	}
	wrefresh(input);    
    }
    strcpy(init, instr);
    if (ch==27) return 0;
    else return 1;
}

int page_one()
{
    char *in;
    int ch, i;
    int pos;
    char tempstr[30], catfile[256];
    
    clear_main_win();
    mvwaddch(optwin.main, 17, 0, ACS_VLINE);
    mvwaddch(optwin.main, 17, 15, ACS_ULCORNER);
    mvwaddch(optwin.main, 18, 15, ACS_VLINE);
    mvwprintw(optwin.main, 18, 2, (char *)gettext("<1> General"));
    mvwhline(optwin.main, 17, 16, ACS_HLINE, 43);
    mvwaddch(optwin.main, 17, 59, ACS_RTEE);
    mvwprintw(optwin.main, 18, 18, (char *)gettext("<2> Other"));
    mvwaddch(optwin.main, 18, 30, ACS_VLINE);

    mvwprintw(optwin.main, 2, 2, (char *)gettext("<F1>   SCSI device: ")); 
    mvwprintw(optwin.main, 2, 29, (char *)gettext("<F2>      Speed: "));    
    mvwprintw(optwin.main, 3, 2, (char *)gettext("<F3>    Image file: "));    
    mvwprintw(optwin.main, 4, 29, (char *)gettext("<I>  Write CD from image: [ ]"));    
    mvwprintw(optwin.main, 5, 2, (char *)gettext("<F4>       Logfile: "));    
    mvwprintw(optwin.main, 6, 2, (char *)gettext("<C>        CD-ROM : "));
    mvwprintw(optwin.main, 7, 2, (char *)gettext("<W>  CD-RW device : "));
    mvwprintw(optwin.main, 8, 2, (char *)gettext("<V>      Volume ID: "));
    mvwprintw(optwin.main, 9, 2, (char *)gettext("<F5>    Blank fast: [ ]"));
    mvwprintw(optwin.main, 9, 29,(char *)gettext("<F6>     Eject when done: [ ]"));
    mvwprintw(optwin.main,10, 2, (char *)gettext("<F7>  Disk at once: [ ]"));
    mvwprintw(optwin.main,10, 29,(char *)gettext("<F8>  Joliet information: [ ]"));
    mvwprintw(optwin.main,11, 2, (char *)gettext("<F9>    Rock Ridge: [ ]"));
    mvwprintw(optwin.main,11, 29, (char *)gettext("<U>    Useful RR attribs: [ ]"));
    mvwprintw(optwin.main,12, 2,(char *)gettext("<F12>  Dummy write: [ ]"));
    mvwprintw(optwin.main,12, 29, (char *)gettext("<L> Show log after write: [ ]"));
    mvwprintw(optwin.main,13, 2,(char *)gettext("<S>  Auto show vol: [ ]"));
    mvwprintw(optwin.main,13, 29, (char *)gettext("<P>                  Pad: [ ]"));
    if ( (strcmp(config.fromimage, "1")==0) || (strcmp(config.dao, "1")==0) ) wattrset(optwin.main, A_BOLD | COLOR_PAIR(9));
    mvwprintw(optwin.main,14, 2,(char *)gettext("<M>  Multi-session: [ ]"));
    wattrset(optwin.main, COLOR_PAIR(2));
    mvwprintw(optwin.main,14, 29, (char *)gettext("<B>            Burnproof: [ ]"));

    optwin.scsidev=derwin(optwin.main, 1, 6, 2, 22);
    wbkgd(optwin.scsidev, COLOR_PAIR(6));
    werase(optwin.scsidev);
    wprintw(optwin.scsidev, "%s", config.scsi);
    wrefresh(optwin.scsidev);

    optwin.speed=derwin(optwin.main, 1, 3, 2, 47);
    wbkgd(optwin.speed, COLOR_PAIR(6));
    werase(optwin.speed);
    wprintw(optwin.speed, "%s", config.speed);
    wrefresh(optwin.speed);

    optwin.tempdir=derwin(optwin.main, 1, 36, 3, 22);
    wbkgd(optwin.tempdir, COLOR_PAIR(6));
    werase(optwin.tempdir);
    wprintw(optwin.tempdir, "%s", config.tempdir);
    wrefresh(optwin.tempdir);

    optwin.logfile=derwin(optwin.main, 1, 36, 5, 22);
    wbkgd(optwin.logfile, COLOR_PAIR(6));
    werase(optwin.logfile);
    wprintw(optwin.logfile, "%s", config.logfile);
    wrefresh(optwin.logfile);

    optwin.cdrom=derwin(optwin.main, 1, 36, 6, 22);
    wbkgd(optwin.cdrom, COLOR_PAIR(6));
    werase(optwin.cdrom);
    wprintw(optwin.cdrom, "%s", config.cdrom);
    wrefresh(optwin.cdrom);

    optwin.cdrwdevice=derwin(optwin.main, 1, 36, 7, 22);
    wbkgd(optwin.cdrwdevice, COLOR_PAIR(6));
    werase(optwin.cdrwdevice);
    wprintw(optwin.cdrwdevice, "%s", config.cdrwdevice);
    wrefresh(optwin.cdrwdevice);

    optwin.volumeid=derwin(optwin.main, 1, 36, 8, 22);
    wbkgd(optwin.volumeid, COLOR_PAIR(6));
    werase(optwin.volumeid);
    wprintw(optwin.volumeid, "%s", config.volumeid);
    wrefresh(optwin.volumeid);

    if ( strncmp(config.blank,"fast",strlen(config.blank))==0)
	mvwprintw(optwin.main, 9, 23, "X");
    if ( strncmp(config.eject,"1",strlen(config.eject))==0)
	mvwprintw(optwin.main, 9, 56, "X");
    if ( strncmp(config.dao,"1",strlen(config.dao))==0)
	mvwprintw(optwin.main, 10, 23, "X");
    if ( strncmp(config.joliet,"1",strlen(config.joliet))==0)
	mvwprintw(optwin.main, 10, 56, "X");
    if ( strncmp(config.rockridge,"1",strlen(config.rockridge))==0)
	mvwprintw(optwin.main, 11, 23, "X");
    if ( strncmp(config.usefulRR,"1",strlen(config.usefulRR))==0)
	mvwprintw(optwin.main, 11, 56, "X");
    if ( strncmp(config.dummy,"1",strlen(config.dummy))==0)
	mvwprintw(optwin.main, 12, 23, "X");
    if ( strncmp(config.showlog,"1",strlen(config.showlog))==0)
	mvwprintw(optwin.main, 12, 56, "X");
    if ( strncmp(config.showvol,"1",strlen(config.showvol))==0)
	mvwprintw(optwin.main, 13, 23, "X");
    if ( strncmp(config.pad,"1",strlen(config.pad))==0)
	mvwprintw(optwin.main, 13, 56, "X");
    if ( strncmp(config.fromimage,"1",strlen(config.fromimage))==0)
	mvwprintw(optwin.main, 4, 56, "X");
    if ( strncmp(config.multi,"1",strlen(config.multi))==0)
	mvwprintw(optwin.main, 14, 23, "X");
    if ( strncmp(config.burnproof,"1",strlen(config.burnproof))==0)
	mvwprintw(optwin.main, 14, 56, "X");
    
    wrefresh(optwin.main);
    
    while ( (ch=mvwgetch(optwin.main,19,59)) != 27 ){
	switch(ch){
	    case KEY_F(1):
		inbox(optwin.scsidev, config.scsi);
		break;
	    case KEY_F(2):
		inbox(optwin.speed, config.speed);
		break;
	    case KEY_F(3):
		inbox(optwin.tempdir, config.tempdir);
		break;
	    case 'I':
	    case 'i':	    
		if ( strncmp(config.fromimage,"1",strlen(config.fromimage))==0){
		    mvwprintw(optwin.main, 4, 56, " ");
		    strcpy(config.fromimage, "0");
		    if ( strcmp(config.dao,"1")!=0 ){
			wattrset(optwin.main, COLOR_PAIR(2));
			mvwprintw(optwin.main,14, 2,(char *)gettext("<M>  Multi-session: [ ]"));
			strcpy(config.multi,"0");
		    }
		} else {
		    mvwprintw(optwin.main, 4, 56, "X");
		    strcpy(config.fromimage,"1");
		    wattrset(optwin.main, A_BOLD | COLOR_PAIR(9));
		    mvwprintw(optwin.main,14, 2,(char *)gettext("<M>  Multi-session: [ ]"));
		    strcpy(config.multi,"0");
		    wattrset(optwin.main, COLOR_PAIR(2));
		}
		break;
	    case KEY_F(4):
		inbox(optwin.logfile, config.logfile);
		break;
	    case 'c':
	    case 'C':
		inbox(optwin.cdrom, config.cdrom);
		break;
	    case 'w':
	    case 'W':
		inbox(optwin.cdrwdevice, config.cdrwdevice);
		break;
	    case KEY_F(5):
		if ( strncmp(config.blank,"fast",strlen(config.blank))==0){
		    mvwprintw(optwin.main, 9, 23, " ");
		    sprintf(config.blank,"all");
		} else {
		    mvwprintw(optwin.main, 9, 23, "X");
		    sprintf(config.blank,"fast");
		}
		break;
	    case KEY_F(6):
		if ( strncmp(config.eject,"1",strlen(config.eject))==0){
		    mvwprintw(optwin.main, 9, 56, " ");
		    strcpy(config.eject, "0");
		} else {
		    mvwprintw(optwin.main, 9, 56, "X");
		    strcpy(config.eject,"1");
		}
		break;
	    case KEY_F(7):
		if ( strncmp(config.dao,"1",strlen(config.dao))==0){
		    mvwprintw(optwin.main, 10, 23, " ");
		    strcpy(config.dao, "0");
		    if ( strcmp(config.fromimage,"1")!=0 ){
			wattrset(optwin.main, COLOR_PAIR(2));
			mvwprintw(optwin.main,14, 2,(char *)gettext("<M>  Multi-session: [ ]"));
			strcpy(config.multi,"0");
		    }
		} else {
		    mvwprintw(optwin.main, 10, 23, "X");
		    strcpy(config.dao,"1");
		    wattrset(optwin.main, A_BOLD | COLOR_PAIR(9));
		    mvwprintw(optwin.main,14, 2,(char *)gettext("<M>  Multi-session: [ ]"));
		    strcpy(config.multi,"0");
		    wattrset(optwin.main, COLOR_PAIR(2));
		}
		break;
	    case KEY_F(8):
		if ( strncmp(config.joliet,"1",strlen(config.joliet))==0){
		    mvwprintw(optwin.main, 10, 56, " ");
		    strcpy(config.joliet, "0");
		} else {
		    mvwprintw(optwin.main, 10, 56, "X");
		    strcpy(config.joliet,"1");
		}
		break;
	    case KEY_F(9):
		if ( strncmp(config.rockridge,"1",strlen(config.rockridge))==0){
		    mvwprintw(optwin.main, 11, 23, " ");
		    strcpy(config.rockridge, "0");
		} else {
		    mvwprintw(optwin.main, 11, 23, "X");
		    strcpy(config.rockridge,"1");
		}
		break;
	    case 'U':
	    case 'u':
		if ( strncmp(config.usefulRR,"1",strlen(config.usefulRR))==0){
		    mvwprintw(optwin.main, 11, 56, " ");
		    strcpy(config.usefulRR, "0");
		} else {
		    mvwprintw(optwin.main, 11, 56, "X");
		    strcpy(config.usefulRR,"1");
		}
		break;
	    case KEY_F(12):
		if ( strncmp(config.dummy,"1",strlen(config.dummy))==0){
		    mvwprintw(optwin.main, 12, 23, " ");
		    strcpy(config.dummy, "0");
		} else {
		    mvwprintw(optwin.main, 12, 23, "X");
		    strcpy(config.dummy,"1");
		}
		break;
	    case 'v':
	    case 'V':
		inbox(optwin.volumeid, config.volumeid);
		break;
	    case 'L':
	    case 'l':
		if ( strncmp(config.showlog,"1",strlen(config.showlog))==0){
		    mvwprintw(optwin.main, 12, 56, " ");
		    strcpy(config.showlog, "0");
		} else {
		    mvwprintw(optwin.main, 12, 56, "X");
		    strcpy(config.showlog,"1");
		}
		break;
	    case 'S':
	    case 's':	    
		if ( strncmp(config.showvol,"1",strlen(config.showvol))==0){
		    mvwprintw(optwin.main, 13, 23, " ");
		    strcpy(config.showvol, "0");
		} else {
		    mvwprintw(optwin.main, 13, 23, "X");
		    strcpy(config.showvol,"1");
		}
		break;
	    case 'P':
	    case 'p':	    
		if ( strncmp(config.pad,"1",strlen(config.pad))==0){
		    mvwprintw(optwin.main, 13, 56, " ");
		    strcpy(config.pad, "0");
		} else {
		    mvwprintw(optwin.main, 13, 56, "X");
		    strcpy(config.pad,"1");
		}
		break;
	    case 'M':
	    case 'm':	    
		if ( (strcmp(config.fromimage,"0")==0) && (strcmp(config.dao,"0")==0) ) {
		    if ( strncmp(config.multi,"1",strlen(config.multi))==0){
			mvwprintw(optwin.main, 14, 23, " ");
			strcpy(config.multi, "0");
		    } else {
			mvwprintw(optwin.main, 14, 23, "X");
			strcpy(config.multi,"1");
		    }
		}
		break;
	    case 'B':
	    case 'b':	    
		if ( strncmp(config.burnproof,"1",strlen(config.burnproof))==0){
		    mvwprintw(optwin.main, 14, 56, " ");
		    strcpy(config.burnproof, "0");
		} else {
		    mvwprintw(optwin.main, 14, 56, "X");
		    strcpy(config.burnproof,"1");
		}
		break;
	    case '2':
	    case KEY_F(10):
		// return main loop
		return ch;
		break;
	    wrefresh(optwin.main);
	}
    }
    return ch;
}

int page_two()
{
    char *in;
    int ch, i;
    int pos;
    char tempstr[30], catfile[256];
    
    clear_main_win();
    mvwaddch(optwin.main, 17, 0, ACS_LTEE);
    mvwhline(optwin.main, 17, 1, ACS_HLINE, 15);
    mvwaddch(optwin.main, 17, 15, ACS_URCORNER);
    mvwaddch(optwin.main, 18, 15, ACS_VLINE);
    mvwprintw(optwin.main, 18, 2, (char *)gettext("<1> General"));
    mvwaddch(optwin.main, 17, 30, ACS_ULCORNER);
    mvwhline(optwin.main, 17, 31, ACS_HLINE, 28);
    mvwaddch(optwin.main, 17, 59, ACS_RTEE);
    mvwprintw(optwin.main, 18, 18, (char *)gettext("<2> Other"));
    mvwaddch(optwin.main, 18, 30, ACS_VLINE);

#ifdef DIC
    wattrset(optwin.main, COLOR_PAIR(2));
#else
    wattrset(optwin.main, A_BOLD | COLOR_PAIR(9));
#endif
    mvwprintw(optwin.main, 2, 2, (char *)gettext("<D> Dic CD drive:"));
    mvwprintw(optwin.main, 2, 31, (char *)gettext("<A>   Dic running auto: [ ]"));
    wattrset(optwin.main, COLOR_PAIR(2));
    mvwprintw(optwin.main, 3, 2, (char *)gettext("<G> Audio directory:"));
    mvwprintw(optwin.main, 4, 2, (char *)gettext("<S>   Grab stereo mode: [ ]"));
    mvwprintw(optwin.main, 4, 31, (char *)gettext("<B>   Bits per channel:"));
    mvwprintw(optwin.main, 5, 2, (char *)gettext("<E> Echo to sound dev.: [ ]"));
#ifdef OGGENC
    wattrset(optwin.main, COLOR_PAIR(2));
#else
    wattrset(optwin.main, A_BOLD | COLOR_PAIR(9));
#endif
    mvwprintw(optwin.main, 6, 2, (char *)gettext("<O>      Encode to Ogg: [ ]"));
    wattrset(optwin.main, COLOR_PAIR(2));
    
#ifdef LAME
    wattrset(optwin.main, COLOR_PAIR(2));
#else
    wattrset(optwin.main, A_BOLD | COLOR_PAIR(9));
#endif
    mvwprintw(optwin.main, 7, 2, (char *)gettext("<M>      Encode to Mp3: [ ]"));
    wattrset(optwin.main, COLOR_PAIR(2));
    
    mvwprintw(optwin.main, 6, 31, (char *)gettext("<H>       High Quality: [ ]"));
    mvwprintw(optwin.main, 7, 31, (char *)gettext("<R>            Bitrate: "));
 
    optwin.dicdisk=derwin(optwin.main, 1, 10, 2, 20);
    wbkgd(optwin.dicdisk, COLOR_PAIR(6));
    werase(optwin.dicdisk);
    wprintw(optwin.dicdisk, "%s", config.dicdisk);
    wrefresh(optwin.dicdisk);

    optwin.audiodir=derwin(optwin.main, 1, 35, 3, 23);
    wbkgd(optwin.audiodir, COLOR_PAIR(6));
    werase(optwin.audiodir);
    wprintw(optwin.audiodir, "%s", config.audiodir);
    wrefresh(optwin.audiodir);

    optwin.bpc=derwin(optwin.main, 1, 3, 4, 55);
    wbkgd(optwin.bpc, COLOR_PAIR(6));
    werase(optwin.bpc);
    wprintw(optwin.bpc, "%s", config.bitsperchn);
    wrefresh(optwin.bpc);

    optwin.bitrate=derwin(optwin.main, 1, 4, 7, 55);
    wbkgd(optwin.bitrate, COLOR_PAIR(6));
    werase(optwin.bitrate);
    wprintw(optwin.bitrate, "%s", config.bitrate);
    wrefresh(optwin.bitrate);
    
    if ( strncmp(config.autodic,"1",strlen(config.autodic))==0){
#ifdef DIC
	wattrset(optwin.main, COLOR_PAIR(2));
#else
	wattrset(optwin.main, A_BOLD | COLOR_PAIR(9));
#endif
	mvwprintw(optwin.main, 2, 56, "X");
    }
    wattrset(optwin.main, COLOR_PAIR(2));

    if ( strncmp(config.stereo,"1",strlen(config.stereo))==0)
	mvwprintw(optwin.main, 4, 27, "X");
    if ( strncmp(config.echosound,"1",strlen(config.echosound))==0)
	mvwprintw(optwin.main, 5, 27, "X");
    if ( strncmp(config.highq,"1",strlen(config.highq))==0)
	mvwprintw(optwin.main, 6, 56, "X");
#ifdef OGGENC	
    if ( strncmp(config.encode,"1",strlen(config.encode))==0){
    	if ( strncmp(config.lame,"1",strlen(config.lame))!=0 )
       	   mvwprintw(optwin.main, 6, 27, "X");
    	else
       	   sprintf(config.encode,"%d",0);
    }
#endif	
   

#ifdef LAME
   if ( strncmp(config.lame,"1",strlen(config.lame))==0){
	if ( strncmp(config.encode,"1",strlen(config.encode))!=0 )
	   mvwprintw(optwin.main, 7, 27, "X");
	else
	   sprintf(config.lame,"%d",0);
    }	
#endif

    wattrset(optwin.main, COLOR_PAIR(2));
    while ( (ch=mvwgetch(optwin.main,19,59)) != 27 ){
	switch(ch){

#ifdef DIC
	    case 'd':
	    case 'D':
		inbox(optwin.dicdisk, config.dicdisk);
		break;
	    case 'A':
	    case 'a':	    
		if ( strncmp(config.autodic,"1",strlen(config.autodic))==0){
		    mvwprintw(optwin.main, 2, 56, " ");
		    strcpy(config.autodic, "0");
		} else {
		    mvwprintw(optwin.main, 2, 56, "X");
		    strcpy(config.autodic,"1");
		}
		break;
#endif
	    case 'g':
	    case 'G':
		inbox(optwin.audiodir, config.audiodir);
		break;
	    case 's':
	    case 'S':
		if ( strncmp(config.stereo,"1",strlen(config.stereo))==0){
		    mvwprintw(optwin.main, 4, 27, " ");
		    sprintf(config.stereo,"0");
		} else {
		    mvwprintw(optwin.main, 4, 27, "X");
		    sprintf(config.stereo,"1");
		}
		break;
	    case 'b':
	    case 'B':
		inbox(optwin.bpc, config.bitsperchn);
		if ( (strcmp(config.bitsperchn,"16")!=0) && (strcmp(config.bitsperchn,"12")!=0) && (strcmp(config.bitsperchn,"8")!=0) ){
		    sprintf(config.bitsperchn, "16");
		    werase(optwin.bpc);
		    mvwprintw(optwin.bpc, 0, 0, "%d", 16);
		    wrefresh(optwin.bpc);
		}
		break;
	    case 'e':
	    case 'E':
		if ( strncmp(config.echosound,"1",strlen(config.echosound))==0){
		    mvwprintw(optwin.main, 5, 27, " ");
		    sprintf(config.echosound,"0");
		} else {
		    mvwprintw(optwin.main, 5, 27, "X");
		    sprintf(config.echosound,"1");
		}
		break;
#ifdef OGGENC		
	    case 'o':
	    case 'O':
		if ( strncmp(config.encode,"1",strlen(config.encode))==0){
		    mvwprintw(optwin.main, 6, 27, " ");
		    sprintf(config.encode,"0");
		} else {
		    mvwprintw(optwin.main, 6, 27, "X");
		    sprintf(config.encode,"1");
		    sprintf(config.lame, "%d", 0);
		    mvwprintw(optwin.main, 7, 27, " ");
		}
		break;
#endif		
#ifdef LAME		
	    case 'm':
	    case 'M':
		if ( strncmp(config.lame,"1",strlen(config.lame))==0){
		    mvwprintw(optwin.main, 7, 27, " ");
		    sprintf(config.lame,"0");
		} else {
		    mvwprintw(optwin.main, 7, 27, "X");
		    sprintf(config.lame,"1");
		    sprintf(config.encode, "%d", 0);
		    mvwprintw(optwin.main, 6, 27, " ");
		}
		break;
#endif		

	    case 'h':
	    case 'H':
		if ( strncmp(config.highq,"1",strlen(config.highq))==0){
		    mvwprintw(optwin.main, 6, 56, " ");
		    sprintf(config.highq,"0");
		} else {
		    mvwprintw(optwin.main, 6, 56, "X");
		    sprintf(config.highq,"1");
		}
		break;
	    case 'r':
	    case 'R':
		inbox(optwin.bitrate, config.bitrate);
		if ( (strcmp(config.bitrate,"128")!=0) && (strcmp(config.bitrate,"192")!=0) && (strcmp(config.bitrate,"256")!=0) ){
		    sprintf(config.bitrate, "192");
		    werase(optwin.bitrate);
		    mvwprintw(optwin.bitrate, 0, 0, "%d", 192);
		    wrefresh(optwin.bitrate);
		}
		break;
	    case '1':
	    case KEY_F(10):
		return ch;
		break;
	wrefresh(optwin.main);
	}
    }
    return ch;
}

void options_window()
{
    char *in;
    int ch, i, page=1;
    
    config=read_conf();
    optwin.main=newwin(20,60,(LINES-20)/2,(COLS-60)/2);
    wbkgd(optwin.main, COLOR_PAIR(2));
    werase(optwin.main);
    box(optwin.main,0,0);
    keypad(optwin.main, TRUE);
    mvwaddch(optwin.main, 0, 2, ACS_RTEE);
    mvwprintw(optwin.main, 0, 3, " %s ", (char *)gettext("Options"));
    waddch(optwin.main, ACS_LTEE);    
    mvwprintw(optwin.main, 19, 2, "( %s )", (char *)gettext("Press F10 to save config or ESC to cancel"));

    ch=page_one();
    while (ch!=27) {
	switch(ch){
	    case '1':
		if ( page!=1 ) {
		    page=1;
		    ch=page_one();
		}
		break;
	    case '2':
		if ( page!=2 ) {
		    page=2;
		    ch=page_two();
		}
		break;
	    case KEY_F(10):
		write_conf(config);
		delwin(optwin.main);
		return;
		break;
	wrefresh(optwin.main);
	}
    }
    delwin(optwin.main);
}
